==> MATT-Bundle-0.5.0.tar.gz: MATT-Bundle-0.5.0/Passwd.pm <==
#!/usr/bin/perl

package MATT::Passwd;

use strict;
use Exporter;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

@ISA   = qw(Exporter);
@EXPORT = qw(DoesSysUserExist Encrypt BackupMasterPasswd VerifyMasterPasswd InstallGroup );
@EXPORT_OK = qw();
$VERSION = 0.5.0;

use MATT::Utility;

sub DoesSysUserExist
{
	my ($user) = @_;

	if ( $user ne lc $user ) {
		die "\tDoesSysUserExist: FAILED: username must be lower case!\n\n";
	};

	if ( getpwnam($user) && getpwnam($user) > 0 )
	{
		return 1;
	} 
	else { return 0 };
};

sub Encrypt
{
	# crypt the plain text password
	my ($pass, $debug) = @_;

BEGIN {
	eval { require Crypt::PasswdMD5 };
	if ($@) {
		die "\nERROR: Crypt::PasswdMD5 is not installed. Try this:\n
perl -MCPAN -e shell
install Crypt::PasswdMD5

    -or on FreeBSD -

cd /usr/ports/security/p5-Crypt-PassMD5
make install clean
";
	} else {
		Crypt::PasswdMD5->import();
	};
};

	my $salt     = rand;
	my $pass_e   = unix_md5_crypt($pass, $salt);

	print "Encrypt: pass_e = $pass_e\n" if $debug;
	return $pass_e;
};

sub BackupMasterPasswd
{
	# Back up master.passwd database
	my $file = "/etc/master.passwd";
   
	SysCmd("sudo cp $file $file.bak");
           
#	use File::Copy;
#	copy($file, "$file.bak")
#		or warn "FATAL: Couldn't back up $file: $!\n";
};

sub VerifyMasterPasswd
{
	# Verify that new master.passwd is larger than prior one
	my ($passwd, $change, $debug) = @_;

	my $new = (stat($passwd))[7];
	my $old = (stat("$passwd.bak"))[7];

	if ( $new == $old ) 
	{
		print "VerifyMasterPasswd: The files are the same size ($new)!\n" if $debug;
	} 
	elsif ( $change eq "grow" && $new > $old )
	{
		print "VerifyMasterPasswd: The file grew ($old to $new) bytes.\n" if $debug;
	} 
	elsif ( $change eq "grow" && $old > $new )
	{
		print "VerifyMasterPasswd: WARNING: The new file is smaller ($new) than $old!\n";
		print "Making archive of master.passwd...";
		ArchiveFile("$passwd.bak");
		print "done.\n";
	} 
	elsif ( $change eq "shrink" && $old > $new )
	{
		if ( ($old - $new) < 300 ) {
			print "VerifyMasterPasswd: The file shrank ($old to $new) bytes.\n" if $debug;
		} else {
			print "VerifyMasterPasswd: WARNING: The file shrank ($old to $new) bytes.\n";
		};
	} else {
		print "VerifyMasterPassword: Yikes, how'd I get here?\n";
	};
};

sub InstallGroup($;$)
{
	my ($group, $gid) = @_;

	my $r = getgrnam($group);

	if ( $r )
	{
		print "group $group (gid: $r) installed\n";
	} else {
		print "installing $group\n";
		if ( $gid )
		{
			SysCmd( "pw groupadd -n $group -g $gid");
		} else {
			SysCmd( "pw groupadd -n $group");
		};
	};
};


1;
__END__

=head1 NAME

        MATT::Passwd.pm - Common Unix Passwd functions

=head1 SYNOPSIS

        Passwd.pm is a grouping of frequently used functions I've written

=head1 DESCRIPTION

        Dependencies

        Crypt::PasswdMD5 -  /usr/ports/security/p5-Crypt-PasswdMD5

=head1 AUTHOR

        Matt Simerson <matt@cadillac.net>

=head1 BUGS

        None known. Report any to author.

=head1 TODO

        Don't export any of the symbols by default. Move all symbols
        to EXPORT_OK and explicitely pull in the required ones in 
        programs that need them.

=head1 SEE ALSO

        http://matt.simerson.net/computing/

=head1 COPYRIGHT

        Copyright 2003, Matt Simerson. All Right Reserved.

=cut


##
# System wide config settings
##
my $home          = "/home";
my $default_group = "user";



####
##  Subroutines
####

sub WriteUserToPasswd
{
	BackupMasterPasswd();
	if (CreateSystemUser($user))
	{
		QuotaSet($home, $user, $quota, $debug);
		VerifyMasterPasswd("/etc/master.passwd", "grow", $debug);
	};
};


sub WriteUserToDB
{
	my $dot   = ParseDotFile (".my.cnf", "[uron]", $debug);
	my $dbv   = MysqlDBVars ( $dot );
	my ($dbh) = MysqlConnect( $dbv );

	$uid = ReadUIDFromDB($dbh, "users", $user);
	if ( ! $uid )
	{
		WriteUserToUsersDB($dbh, $user);
		$uid = ReadUIDFromDB($dbh, "users", $user);
		die "FATAL: Can't read uid from database!\n" if (! $uid);
	}
	else
	{
		print "WriteUserToDB: $user already exists in users!\n";
	};

	if ( ! ReadShadowFromDB($dbh, "pw_shadow", $uid ) )
	{
		$shadow = WriteUserToShadowDB($dbh, $uid, $pass);
	} else {
		print "WriteUserToDB: uid $uid already exists in pw_shadow!\n";
	};

	if ( ! ReadUIDFromDB($dbh, "pw_passwd", $user) )
	{
		WriteUserToPasswdDB($dbh, $uid, $shadow);
	}
	else
	{
		print "WriteUserToDB: uid $uid already exists in pw_passwd!\n";
	};
};

sub WriteUserToPasswdDB
{
	# Write passwd information for accounts into the SQL table.
	# user, shadow, uid, gid, quota, gecos, dir, shell, expire to pw_passwd table
	my ($dbh, $uid, $shadow) = @_;
	my $gid;

	if (! $group) { $gid = getgrnam("user") } else { $gid = getgrnam($group) };

	my $query = "INSERT INTO pw_passwd SET user='$user', passwd='$shadow', uid='$uid', gid='$gid', quota='$quota', gecos='$name', dir='$home_dir', shell='/sbin/nologin'";
	#print "WritePasswd Query: $query\n" if ($debug);
	my $sth   = MysqlQuery($dbh, $query);
	$sth->finish;

	$uid = ReadUIDFromDB($dbh, "pw_passwd", $user);
	die "FATAL: Can't read uid from database!\n" if (! $uid);
};

sub WriteUserToShadowDB
{
	#   write passwd & crypted passwd into uron.pw_shadow
	my ($dbh, $uid, $pass) = @_;

	my $shadow = Encrypt($pass, $debug);

	my $query = "INSERT INTO pw_shadow SET uid='$uid', passwd='$shadow', passwd_clear='$pass'";
	print "WriteShadow Query: $query\n" if ($debug);
	my $sth   = MysqlQuery($dbh, $query);
	$sth->finish;

	$uid = ReadShadowFromDB($dbh, "pw_shadow", $uid);
	die "FATAL: Can't read passwd from shadow database!\n" if (! $uid);

	return $shadow;
};

sub ReadShadowFromDB
{
	my ($dbh, $table, $uid) = @_;

	my $query = "SELECT passwd_clear FROM $table WHERE uid='$uid'";
	#print "ReadShadow query: $query\n" if $debug;
	my $sth   = MysqlQuery($dbh, $query);

	my $r     = $sth->fetchrow_arrayref;
	$sth->finish;
	print "ReadShadow: uid $uid, passwd: $r->[0]\n" if $debug;
	if ( $r->[0] ) {
		return $r->[0];
	} else {
		return 0;
	};
};

sub ReadUIDFromDB
{
	my ($dbh, $table, $user) = @_;

	my $query = "SELECT uid FROM $table WHERE user='$user' AND deleted='0'";
	#print "ReadUID query: $query\n" if $debug;
	my $sth   = MysqlQuery($dbh, $query);

	#my $rv    = $sth->rows;
	#print "rows: $rv\n" if $debug;

	my $r     = $sth->fetchrow_arrayref;
	$sth->finish;
	if ( $r->[0] ) {
		print "ReadUID $table: user: $user, uid: $r->[0]\n" if $debug;
		return $r->[0];
	} else {
		print "ReadUID $table: user: $user, uid: FAILED\n" if $debug;
		return 0;
	};
};

sub WriteUserToUsersDB
{
	#   write user name into uron.users table
	my ($dbh, $user) = @_;

	my $query = "INSERT INTO users SET user='$user', deleted='0'";
	print "WriteUser Query: $query\n" if ($debug);
	my $sth   = MysqlQuery($dbh, $query);
	$sth->finish;
};

sub ProcessShell
{
	use Getopt::Std;
	my @optlist;

	# Accept parameters for the account via flags on the command line
	getopts('f:g:l:n:u:p:q:v', \%opts);

	$debug = 1 if $opts{'v'};
	print "debugging enabled\n" if $opts{'v'};

	if ( $opts{'e'} ) { 
		$expire = $opts{'e'};
		push @optlist, "Expire: $expire\n";
	};

	if ( $opts{'f'} ) { 
		$fname  = $opts{'f'};
		push @optlist, "First Name: $fname\n";
	};

	if ( $opts{'g'} ) { 
		$group  = $opts{'g'};
		push @optlist, "Group: $group\n";
	} else { Usage() };

	if ( $opts{'l'} ) { 
		$lname  = $opts{'l'};
		push @optlist, "Last Name: $lname\n";
	};

	if ( $opts{'n'} ) { 
		$name   = $opts{'n'};
		push @optlist, "Name: $name\n";
	};

	if ( $opts{'p'} ) { 
		$pass   = $opts{'p'};
		push @optlist, "Password: *******\n";
	} else { Usage() };

	if ( $opts{'u'} ) { 
		$user   = $opts{'u'};
		push @optlist, "Username: $user\n";
	} else { Usage() };

	if ( $opts{'q'} ) {
		$quota  = $opts{'q'};
		push @optlist, "Quota: $quota\n";
	} else { Usage() };

	$home_dir = "$home/$user";

	if (! $name) { $name = "$fname $lname" };

	LogFileAppend("user.log", "user_add: opts", @optlist );
	LogFileAppend("user.log", "user_add: args", @ARGV) if $ARGV[0];
};

sub Usage
{
	print "    ***    ***    Create User    ***     ***

     -g  group    Group
     -f  fname    First Name
     -l  lname    Last Name
     -n  name     Full/Company Name
     -p  pass     password (plaintext)
     -q  quota    Disk Quota (in MB)
     -u  user     user name
     -v  verbose  show debugging & status messages
\n";

	LogFileAppend("user.log", "user_add:", @ARGV);
	die "\n";
};

==> MATT-Bundle-0.6.0.tar.gz: MATT-Bundle-0.6.0/Passwd.pm <==
#!/usr/bin/perl

package MATT::Passwd;

use strict;
use Exporter;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

@ISA   = qw(Exporter);
@EXPORT = qw(DoesSysUserExist Encrypt BackupMasterPasswd VerifyMasterPasswd InstallGroup );
@EXPORT_OK = qw();
$VERSION = 0.6.0;

use MATT::Utility;

sub DoesSysUserExist
{
	my ($user) = @_;

	if ( $user ne lc $user ) {
		die "\tDoesSysUserExist: FAILED: username must be lower case!\n\n";
	};

	if ( getpwnam($user) && getpwnam($user) > 0 )
	{
		return 1;
	} 
	else { return 0 };
};

sub Encrypt
{
	# crypt the plain text password
	my ($pass, $debug) = @_;

BEGIN {
	eval { require Crypt::PasswdMD5 };
	if ($@) {
		die "\nERROR: Crypt::PasswdMD5 is not installed. Try this:\n
perl -MCPAN -e shell
install Crypt::PasswdMD5

    -or on FreeBSD -

cd /usr/ports/security/p5-Crypt-PassMD5
make install clean
";
	} else {
		Crypt::PasswdMD5->import();
	};
};

	my $salt     = rand;
	my $pass_e   = unix_md5_crypt($pass, $salt);

	print "Encrypt: pass_e = $pass_e\n" if $debug;
	return $pass_e;
};

sub BackupMasterPasswd
{
	# Back up master.passwd database
	my $file = "/etc/master.passwd";
   
	SysCmd("sudo cp $file $file.bak");
           
#	use File::Copy;
#	copy($file, "$file.bak")
#		or warn "FATAL: Couldn't back up $file: $!\n";
};

sub VerifyMasterPasswd
{
	# Verify that new master.passwd is larger than prior one
	my ($passwd, $change, $debug) = @_;

	my $new = (stat($passwd))[7];
	my $old = (stat("$passwd.bak"))[7];

	if ( $new == $old ) 
	{
		print "VerifyMasterPasswd: The files are the same size ($new)!\n" if $debug;
	} 
	elsif ( $change eq "grow" && $new > $old )
	{
		print "VerifyMasterPasswd: The file grew ($old to $new) bytes.\n" if $debug;
	} 
	elsif ( $change eq "grow" && $old > $new )
	{
		print "VerifyMasterPasswd: WARNING: The new file is smaller ($new) than $old!\n";
		print "Making archive of master.passwd...";
		ArchiveFile("$passwd.bak");
		print "done.\n";
	} 
	elsif ( $change eq "shrink" && $old > $new )
	{
		if ( ($old - $new) < 300 ) {
			print "VerifyMasterPasswd: The file shrank ($old to $new) bytes.\n" if $debug;
		} else {
			print "VerifyMasterPasswd: WARNING: The file shrank ($old to $new) bytes.\n";
		};
	} else {
		print "VerifyMasterPassword: Yikes, how'd I get here?\n";
	};
};

sub InstallGroup($;$)
{
	my ($group, $gid) = @_;

	my $r = getgrnam($group);

	if ( $r )
	{
		print "InstallGroup: group $group (gid: $r) installed\n";
	}
	else
	{
		print "InstallGroup: installing $group\n";

		my $pw = FindTheBin("pw");

		if ( $gid )
		{
			SysCmd( "$pw groupadd -n $group -g $gid");
		} else {
			SysCmd( "$pw groupadd -n $group");
		};
	};
};

sub InstallUser($;$$)
{
	my ($user, $shell, $homedir) = @_;

	my $r  = getpwnam($user);

	$shell = "/sbin/nologin" if (!$shell);

	if ( getpwnam($user) )
	{
		print "user $user (uid: $r) already exists\n";
	}
	else
	{
		print "installing $user\n";

		my $pw = FindTheBin("pw");

		if (  $user eq "vpopmail" )
		{
			SysCmd("$pw useradd -n $user -u 89 -g 89 -c Vpopmail-Master -d $homedir -s $shell");
		}
		elsif (  $user eq "clamav" )
		{
			my $gid = getgrnam("clamav");
			SysCmd( "$pw useradd -n $user -g $gid -d /nonexistent -c Clam-Antivirus -s $shell");
		}
		else
		{
			if ($homedir)
			{
				SysCmd( "$pw useradd -n $user -d $homedir -s $shell");
			} else {
				SysCmd( "$pw useradd -n $user -s $shell");
			};
		};
	};
};

1;
__END__

=head1 NAME

        MATT::Passwd.pm - Common Unix Passwd functions

=head1 SYNOPSIS

        Passwd.pm is a grouping of frequently used functions I've written

=head1 DESCRIPTION

        Dependencies

        Crypt::PasswdMD5 -  /usr/ports/security/p5-Crypt-PasswdMD5

=head1 AUTHOR

        Matt Simerson <matt@cadillac.net>

=head1 BUGS

        None known. Report any to author.

=head1 TODO

        Don't export any of the symbols by default. Move all symbols
        to EXPORT_OK and explicitely pull in the required ones in 
        programs that need them.

=head1 SEE ALSO

        http://matt.simerson.net/computing/

=head1 COPYRIGHT

        Copyright 2003, Matt Simerson. All Right Reserved.

=cut
